import processing.core.*; 
import processing.xml.*; 

import codeanticode.gsvideo.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class scanner_camera extends PApplet {

/*
  SCANNER CAMERA
  by ROMAN KOGAN
  romwell@gmail.com
  http://www.ic.sunysb.edu/Stu/rkogan
  
  Arrows change scan direction
  Space re-starts scan
  Enter changes camera resolutions
  Click to change camera settings

  Files are saved at the end of the scan into the program directory
  
  Edit settings.xml to add resolution modes
*/



String BOILER_TEXT="Webcam Scanner Camera - Roman Kogan - romwell@gmail.com - http://www.ic.sunysb.edu/Stu/rkogan";
int[][] res_modes = {{320,240,1000,500}};
String[] res_mode_names = {"Default"};
int RES_MODE = 0;
int MODE = 0;
int STEP_DIR = 1;
int CAM_W, CAM_H;
int H_CAM_OFFSET, V_CAM_OFFSET;
int W, H;
GSCapture m;

int step = 0;
int frame_counter=0;



public void setup(){  
  loadSettings();
  CAM_W = res_modes[RES_MODE][0];
  CAM_H = res_modes[RES_MODE][1];
  m = new GSCapture(this, CAM_W,CAM_H); //start a capture (usually at 320 x 240)  
  setMode(MODE, STEP_DIR); 
}

public void loadSettings()
{
 if(sketchFile("settings.xml").exists()){
    XMLElement xml = new XMLElement(this, sketchPath("settings.xml"));
    int numModes = xml.getChildCount();
    res_modes = new int[numModes][4];
    res_mode_names = new String[numModes];    
    for (int i = 0; i < numModes; i++) {
      XMLElement kid = xml.getChild(i);
      res_modes[i][0] = kid.getIntAttribute("cam_width"); 
      res_modes[i][1] = kid.getIntAttribute("cam_height"); 
      res_modes[i][2] = kid.getIntAttribute("horizontal_scan_width"); 
      res_modes[i][3] = kid.getIntAttribute("vertical_scan_height");     
      res_mode_names[i] = kid.getContent();
    }  
 }
}


public void changeResolutionMode(int resMode)
{
  if ((resMode % res_modes.length) != RES_MODE)
  {
    RES_MODE = resMode % res_modes.length;
    setup();
  }
}

public void setMode(int mode, int step_dir)
{
  MODE = mode % 2;
  H_CAM_OFFSET = CAM_W*CAM_H/2;
  V_CAM_OFFSET = CAM_W/2;
  STEP_DIR = step_dir;
  if (MODE == 0)
  { 
    W = res_modes[RES_MODE][2]; ; 
    H = CAM_H; 
  }
  else 
  { 
    W = CAM_W; 
    H = res_modes[RES_MODE][3]; 
  }
  step = getStartStep(mode, step_dir);
  size(W,H);  
  frame.setSize(W,H); 
  frame.setTitle(BOILER_TEXT + " :|:  RES MODE: " + res_mode_names[RES_MODE]);
  background(0);
  loadPixels();
}

public int getStartStep(int mode, int step_dir)
{
  if (mode == 0)
  { 
    if (step_dir < 0) {return W - 1;} else {return 0;}
  }
  else 
  { 
    if (step_dir < 0) {return H - 1;} else {return 0;}
  }
 
}

public void draw(){
  if (m.available()){
  m.read();//update the camera view 
  int[] img = m.pixels;   
  if (MODE == 0){
    for(int i=0;i<H;i++){ //loop through all the pixels
      pixels[step+i*W] = img[V_CAM_OFFSET+i*CAM_W]; //draw each pixel to the screen
    }
    step = (step + STEP_DIR + W) % W;    
  }
  else
  {    
    int H_SCREEN_OFFSET = W*step;
    for(int i=0;i<W;i++){ //loop through all the pixels
      pixels[i+H_SCREEN_OFFSET] = img[i+H_CAM_OFFSET]; //draw each pixel to the screen
    }
    step = (step + STEP_DIR + H) % H;   
  } 
  updatePixels();
  if (step == getStartStep(MODE, STEP_DIR))
  {
    saveImage();    
  }
  }
}

public void mousePressed(){
//    m.settings();//click the window to get the settings
}


public void keyPressed() {
  if (key == CODED) {
    if (keyCode == DOWN) {
      setMode(1,1);
    } 
    else if (keyCode == UP) {
       setMode(1,-1);
    } 
    else if (keyCode == RIGHT) {
       setMode(0,1);  
    } 
    else if (keyCode == LEFT) {
       setMode(0,-1);
    }    
  }
  else if (key == ' ')
  {
    setMode(MODE, STEP_DIR);
  } 
  else if (((int)key == 10)||((int)key == 13))
  {
    changeResolutionMode(RES_MODE + 1);
  } 
}


public String pad(String S, int L)
{
 String ans = S;
 while (ans.length() < L) {ans = '0'+ans;}
 return ans;
}

public String getFileName()
{
  return "scan_photo_"+pad(""+frame_counter,4)+".JPG";  
}

public void saveImage()
{
  while (sketchFile(getFileName()).exists()) {frame_counter++;}
  save(getFileName());
}    


  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#DFDFDF", "scanner_camera" });
  }
}
